find_package(Git REQUIRED)
set(GITREV_FILE ${CMAKE_BINARY_DIR}/${OG_PNAME}_git_rev.h)
set(GITREV_TMP ${CMAKE_BINARY_DIR}/git-rev-tmp.h)
if(NOT DEFINED OG_BUILD_KIT_CMAKE_DIR)
    message(FATAL_ERROR "include this module after OG_BUILD_KIT_CMAKE_DIR definition!")
endif()
configure_file(${OG_BUILD_KIT_CMAKE_DIR}/git-rev-gen.sh ${CMAKE_BINARY_DIR}/git-rev-gen.sh COPYONLY)
if (DEFINED WIN32)
add_custom_command(
    OUTPUT ${GITREV_TMP}
    COMMAND ${CMAKE_COMMAND} -E copy ${OG_BUILD_KIT_CMAKE_DIR}/in/null_git_rev.h ${GITREV_TMP}
    COMMAND ${CMAKE_COMMAND} -E copy_if_different ${GITREV_TMP} ${GITREV_FILE}
    COMMAND ${CMAKE_COMMAND} -E remove ${GITREV_TMP}
)
else()
add_custom_command(
    OUTPUT ${GITREV_TMP}
    COMMAND ${CMAKE_BINARY_DIR}/git-rev-gen.sh >> ${GITREV_TMP}
    COMMAND ${CMAKE_COMMAND} -E copy_if_different ${GITREV_TMP} ${GITREV_FILE}
    COMMAND ${CMAKE_COMMAND} -E remove ${GITREV_TMP}
    WORKING_DIRECTORY ${CMAKE_SOURCE_DIR}
    VERBATIM
)
endif()
#set(OG_PBUILDINFO_SRC_IN ${OG_BUILD_KIT_CMAKE_DIR}/in/og_buildinfo.c.in)
#set(OG_PBUILDINFO_HDR_IN ${OG_BUILD_KIT_CMAKE_DIR}/in/og_buildinfo.h.in)
set(OG_PBUILDINFO_SRC ${CMAKE_BINARY_DIR}/${OG_PNAME}_buildinfo.cpp)
set(OG_PBUILDINFO_HDR ${CMAKE_BINARY_DIR}/${OG_PNAME}_buildinfo.hpp)
configure_file(${OG_BUILD_KIT_CMAKE_DIR}/in/og_buildinfo.c.in ${OG_PBUILDINFO_SRC})
configure_file(${OG_BUILD_KIT_CMAKE_DIR}/in/og_buildinfo.h.in ${OG_PBUILDINFO_HDR})
add_custom_target(og_buildinfo_phony DEPENDS ${GITREV_TMP})
add_library(og_buildinfolib
    ${OG_PBUILDINFO_SRC})
target_include_directories(og_buildinfolib
    PUBLIC ${CMAKE_BINARY_DIR})
add_dependencies(og_buildinfolib og_buildinfo_phony)

function(_git_find_closest_git_dir _start_dir _git_dir_var)
    set(cur_dir "${_start_dir}")
    set(git_dir "${_start_dir}/.git")
    while(NOT EXISTS "${git_dir}")
        # .git dir not found, search parent
        set(git_previous_parent "${cur_dir}")
        get_filename_component(cur_dir "${cur_dir}" DIRECTORY)
        if(cur_dir STREQUAL git_previous_parent)
            set(${_git_dir_var} "" PARENT_SCOPE)
            return()
        endif()
        set(git_dir "${cur_dir}/.git")
    endwhile()
    set(${_git_dir_var} "${git_dir}" PARENT_SCOPE)
endfunction()

# unused
function(get_git_head_revision _refspecvar _hashvar)
    _git_find_closest_git_dir("${CMAKE_CURRENT_SOURCE_DIR}" GIT_DIR)
    if(NOT "${GIT_DIR}" STREQUAL "")
        file(RELATIVE_PATH _relative_to_source_dir "${CMAKE_SOURCE_DIR}" "${GIT_DIR}")
    endif()
    if("${GIT_DIR}" STREQUAL "")
        set(${_refspecvar} "GITDIR-NOTFOUND" PARENT_SCOPE)
        set(${_hashvar} "GITDIR-NOTFOUND" PARENT_SCOPE)
        return()
    endif()

    # check if the current source dir is a git submodule or a worktree.
    # In both cases .git is a file instead of a directory.
    if(NOT IS_DIRECTORY ${GIT_DIR})
        execute_process(
            COMMAND "${GIT_EXECUTABLE}" rev-parse --show-superproject-working-tree
            WORKING_DIRECTORY "${CMAKE_CURRENT_SOURCE_DIR}"
            OUTPUT_VARIABLE out
            ERROR_QUIET OUTPUT_STRIP_TRAILING_WHITESPACE)
        if(NOT "${out}" STREQUAL "")
            # in a submodule
            file(READ ${GIT_DIR} submodule)
            string(REGEX REPLACE "gitdir: (.*)$" "\\1" GIT_DIR_RELATIVE
                ${submodule})
            string(STRIP ${GIT_DIR_RELATIVE} GIT_DIR_RELATIVE)
            get_filename_component(SUBMODULE_DIR ${GIT_DIR} PATH)
            get_filename_component(GIT_DIR ${SUBMODULE_DIR}/${GIT_DIR_RELATIVE} ABSOLUTE)
            set(HEAD_SOURCE_FILE "${GIT_DIR}/HEAD")
        else()
            # in a worktree
            file(READ ${GIT_DIR} worktree_ref)
            string(REGEX REPLACE "gitdir: (.*)$" "\\1" git_worktree_dir ${worktree_ref})
            string(STRIP ${git_worktree_dir} git_worktree_dir)
            _git_find_closest_git_dir("${git_worktree_dir}" GIT_DIR)
            set(HEAD_SOURCE_FILE "${git_worktree_dir}/HEAD")
        endif()
    else()
        set(HEAD_SOURCE_FILE "${GIT_DIR}/HEAD")
    endif()
    set(GIT_DATA "${CMAKE_CURRENT_BINARY_DIR}/CMakeFiles/git-data")
    if(NOT EXISTS "${GIT_DATA}")
        file(MAKE_DIRECTORY "${GIT_DATA}")
    endif()
    if(NOT EXISTS "${HEAD_SOURCE_FILE}")
        return()
    endif()
    set(HEAD_FILE "${GIT_DATA}/HEAD")
    configure_file("${HEAD_SOURCE_FILE}" "${HEAD_FILE}" COPYONLY)
    configure_file("${_gitdescmoddir}/gitrev_desc.cmake.in" "${GIT_DATA}/grabref.cmake" @ONLY)
    include("${GIT_DATA}/grabref.cmake")
    set(${_refspecvar} "${HEAD_REF}" PARENT_SCOPE)
    set(${_hashvar} "${HEAD_HASH}" PARENT_SCOPE)
endfunction()
