macro(og_project_start)
if(NOT ${CMAKE_PROJECT_NAME} STREQUAL ${OG_PNAME})
    message(FATAL_ERROR "project name [${CMAKE_PROJECT_NAME}] in CMakelist.txt is different from name [${OG_PNAME}] in versions.yaml!")
endif()
if(NOT ${CMAKE_PROJECT_VERSION} VERSION_EQUAL ${OG_PVER})
    message(FATAL_ERROR "project version [${CMAKE_PROJECT_VERSION}] in CMakelist.txt is different from version [${OG_PVER}] in versions.yaml!")
endif()
message("Project: ${OG_PNAME}-${OG_PVER}")
set(OG_PPREFIX ${OG_PNAME}/${OG_PVER})
set(OG_PTGTNAME ${OG_PNAME}-targets)
set(OG_PNAMESPACE ${OG_PNAME}::)
set(OG_INSTALL_PREFIX  ${DISTROOT})
set(CMAKE_INSTALL_PREFIX ${OG_INSTALL_PREFIX} CACHE PATH "hardcoded" FORCE)
message("Install path: ${CMAKE_INSTALL_PREFIX}/${OG_PPREFIX}")
if(NOT IS_DIRECTORY ${CMAKE_INSTALL_PREFIX})
file(MAKE_DIRECTORY ${CMAKE_INSTALL_PREFIX})
endif()
set(CMAKE_BUILD_RPATH_USE_ORIGIN TRUE)
set(CMAKE_INSTALL_RPATH_USE_LINK_PATH TRUE)
list(APPEND CMAKE_INSTALL_RPATH $ORIGIN)
list(APPEND CMAKE_INSTALL_RPATH $ORIGIN/../lib)
# assume the install path is /some_root/pkgname-0.0.1/lib
# below enable lib search under /some_root/lib 
list(APPEND CMAKE_INSTALL_RPATH $ORIGIN/../../lib)

set(CMAKE_RUNTIME_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/bin)
set(CMAKE_ARCHIVE_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/lib)
set(CMAKE_LIBRARY_OUTPUT_DIRECTORY ${CMAKE_BINARY_DIR}/lib)
set(OG_PCONFIG_IN_PATH ${OG_BUILD_KIT_CMAKE_DIR}/in/config.cmake.in)
set(CMAKE_CXX_STANDARD 20)
set(CMAKE_CXX_STANDARD_REQUIRED True)
set(CMAKE_EXPORT_COMPILE_COMMANDS 1)
set(CMAKE_POSITION_INDEPENDENT_CODE ON)
#ogbk_find_all_packages()
include(${OG_BUILD_KIT_CMAKE_DIR}/common/buildinfo.cmake)
include_directories(${PROJECT_SOURCE_DIR}/include)

endmacro()

macro(og_project_end)
get_property(OG_TARGET_LIST GLOBAL PROPERTY OG_PTARGETS)
get_property(OG_TARGET_LIST_NS GLOBAL PROPERTY OG_PTARGETS_NS)
message("Following og cmake targets will be installed (with namespace=${OG_PNAMESPACE}):")
message("${OG_TARGET_LIST}")
if ("${OG_TARGET_LIST}" STREQUAL "")
configure_file(${OG_BUILD_KIT_CMAKE_DIR}/in/null_targets.cmake.in ${CMAKE_BINARY_DIR}/${OG_PTGTNAME}.cmake)
install(FILES
${CMAKE_BINARY_DIR}/${OG_PTGTNAME}.cmake
DESTINATION ${OG_PPREFIX}/cmake
)
else()
install(EXPORT ${OG_PTGTNAME}
NAMESPACE ${OG_PNAMESPACE}
FILE ${OG_PTGTNAME}.cmake
DESTINATION ${OG_PPREFIX}/cmake
)
endif()

include(CMakePackageConfigHelpers)
if (NOT DEFINED OG_PPREFIX_INCLUDE)
set(OG_PPREFIX_INCLUDE ${OG_PPREFIX}/include)
endif()
configure_package_config_file(${OG_PCONFIG_IN_PATH}
    "${CMAKE_CURRENT_BINARY_DIR}/${OG_PNAME}-config.cmake"
    INSTALL_DESTINATION ${OG_PPREFIX}/cmake
    PATH_VARS OG_PPREFIX OG_PPREFIX_INCLUDE
    NO_SET_AND_CHECK_MACRO
    NO_CHECK_REQUIRED_COMPONENTS_MACRO
    )
write_basic_package_version_file(
    ${CMAKE_CURRENT_BINARY_DIR}/${OG_PNAME}-config-version.cmake
    VERSION "${OG_PVER}"
    COMPATIBILITY SameMinorVersion 
    )
install(FILES
${CMAKE_CURRENT_BINARY_DIR}/${OG_PNAME}-config.cmake
${CMAKE_CURRENT_BINARY_DIR}/${OG_PNAME}-config-version.cmake
DESTINATION ${OG_PPREFIX}/cmake
)
#install(FILES
#${PROJECT_SOURCE_DIR}/versions.yaml
#DESTINATION ${OG_PPREFIX}
#)
if (OG_INSTALL_BUILDINFO)
  install(FILES
  ${CMAKE_CURRENT_BINARY_DIR}/compile_commands.json
  ${OG_PBUILDINFO_SRC}
  ${GITREV_FILE}
  DESTINATION ${OG_PPREFIX}/buildinfo
  )
  install(FILES
  ${OG_WORKSPACE_FILE}
  RENAME ogbk_buildinfo_$<CONFIG>.json
  DESTINATION ${OG_PPREFIX}/buildinfo)
endif()
if (OG_INSTALL_INCLUDE)
  if(IS_DIRECTORY ${PROJECT_SOURCE_DIR}/include)
      install(DIRECTORY ${PROJECT_SOURCE_DIR}/include
      DESTINATION ${OG_PPREFIX}
      USE_SOURCE_PERMISSIONS
      FILES_MATCHING
      PATTERN "*.ipp" 
      PATTERN "*.hpp"
      PATTERN "*.h"
      PATTERN "*.inl"
      PATTERN "*.impl"
      )
  endif()
endif()
if(IS_DIRECTORY ${PROJECT_SOURCE_DIR}/cmake)
    install(DIRECTORY ${PROJECT_SOURCE_DIR}/cmake
    DESTINATION ${OG_PPREFIX}
    USE_SOURCE_PERMISSIONS)
endif()
# required if target is used from build tree 
if (NOT "${OG_TARGET_LIST}" STREQUAL "")
export(EXPORT ${OG_PTGTNAME} 
    FILE "${CMAKE_CURRENT_BINARY_DIR}/${OG_PTGTNAME}.cmake"
    )
endif()
endmacro()
