#pragma once
// TODO: c-style interface
#include <vector>
#include <functional>
#include <linalg/type.hpp>
#include <pcommon/macro.hpp>
using objfunc_t = std::function<double(dynvec_t)>;
using jacfunc_t = std::function<dynvec_t(dynvec_t)>;
struct constraint_t {
  explicit constraint_t(objfunc_t&& f, jacfunc_t&&jac):f_(std::move(f)), jac_(std::move(jac)) {
  }

  objfunc_t f_;
  jacfunc_t jac_;
};

/* cost/dcost and constraints are user provided
 * */
struct minimz_i_t {
  objfunc_t func = nullptr;
  jacfunc_t jac = nullptr;
  dynvec_t x0;
  std::vector<constraint_t> cons;
  int maxiter;
  double ftol;
  double jac_eps;
};

struct slsqp_ws_t {
  explicit slsqp_ws_t(minimz_i_t&& input): i_(input) {
    majiter_ = i_.maxiter-1;
    majiter_prev_ = 0;
  }

  void minimize();

  bool success() const {
    return run_ && (mode_ == 0);
  }

  double error() const {
    return fx_;
  }

  int ret() const {
    return mode_;
  }
  const char* retstr() const;
protected:
  bool run_ = false;
  double fx_ = DNAN; // error, or "fun" in scipy.slsqp.py
  minimz_i_t i_;
  int mode_ = 0;
  int majiter_;
  int majiter_prev_;
  dynvec_t g_, c_, x_;
  int la_, n_, m_, meq_, mieq_;
  dynmat_t a_;

  // stored in g_
  void eval_g_inplace();
  // stored in c_
  void eval_con_inplace();
  // stored in a_
  void eval_con_normals_inplace();
};

