#pragma once
#include <common/dt.hpp>
#include <sm/sm.hpp>
#include <memory>
namespace pricer {
  struct ttm_t {
    option_term& term;
    double days;
    tp::tp_t ts;
  };
  using ttm_output_func_t = std::function<void(const ttm_t&)>;

  struct ttm_init_t {
    virtual ~ttm_init_t() = default;
    explicit ttm_init_t(std::string m, double ss, ttm_output_func_t&& cb):
      method(m), min_step(ss), callback(std::move(cb)) {}
    std::string method;
    duration::ds_t min_step;
    std::optional<duration::ds_t> floor;
    ttm_output_func_t callback;
    using pointer_type = std::shared_ptr<ttm_init_t>;
  };

  struct fixed_discount_ttm_init_t : ttm_init_t {
    inline static std::string METHOD = "fixed";
    explicit fixed_discount_ttm_init_t(double d,
        double ss,
        ttm_output_func_t&& cb): ttm_init_t(METHOD, ss, std::move(cb)), discount(d) {}
    double discount;
  };

  struct ttm_clock_impl_t;
  struct ttm_clock_t {
    ttm_clock_t(ttm_init_t::pointer_type init);
    virtual void on_timer(tp::tp_t now) = 0;
    virtual void on_param_update() {}
    virtual void add_term(option_term&) = 0;

    using pointer_type = std::shared_ptr<ttm_clock_t>;
    static ttm_clock_t::pointer_type make(ttm_init_t::pointer_type init);
  protected:
    std::unique_ptr<ttm_clock_impl_t> pimpl_;
  };

} // namespace pricer
