#include "optmz/minmz.hpp"
#include "cslsqp.h"
#include <Eigen/Eigen>
#include <cmath>
#include "pcommon/macro.hpp"

void fillv(dynvec_t &vec, double v) {
  for (size_t i = 0; i < vec.size(); ++i) {
    vec[i] = v;
  }
}

void slsqp_ws_t::minimize() {
	if (!i_.func || !i_.jac) {
    // i_.func
  }

  if (run_) {
    return;
  }
  run_ = true;
  x_ = i_.x0;
  // assume no bounds
  //

  // only support mieq, i.e., NoLinearConstraint in SciPy, for now.
  meq_ = 0;
  mieq_ = i_.cons.size();
  m_ = meq_ + mieq_;
  la_ = std::max(1, m_);
  n_ = x_.size();
  int n1 = n_ + 1;
  int mineq = m_ - meq_ + n1 + n1;
  int len_w = (3 * n1 + m_) * (n1 + 1) + (n1 - meq_ + 1) * (mineq + 2) +
              2 * mineq + (n1 + mineq) * (n1 - meq_) + 2 * meq_ + n1 +
              ((n_ + 1) * n_) / 2 + 2 * m_ + 3 * n_ + 3 * n1 + 1;
  int len_jw = mineq;
  dynvec_t w = Eigen::VectorXd::Zero(len_w);
  dynveci_t jw = Eigen::VectorXi::Zero(len_jw);
  dynvec_t xl = Eigen::VectorXd::Constant(n_, DNAN);
  dynvec_t xu = Eigen::VectorXd::Constant(n_, DNAN);
  double acc = i_.ftol;
  double alpha = 0.0;
  double f0 = 0.0;
  double gs = 0.0;
  double h1 = 0.0;
  double h2 = 0.0;
  double h3 = 0.0;
  double h4 = 0.0;
  double t = 0.0;
  double t0 = 0.0;
  double to1 = 0.0;
  int iexact = 0;
  int incons = 0;
  int ireset = 0;
  int itermx = 0;
  int line = 0;
  // scipy reuse the same name "n1" for two seemly different variables
  n1 = 0;
  int n2 = 0;
  int n3 = 0;
  fx_ = i_.func(x_);
  eval_g_inplace();
  c_ = Eigen::VectorXd(m_);
  EXPECT(c_.size() == m_, "slsqp.c invalid sz %lu", c_.size())
  a_ = Eigen::MatrixXd(la_, n_+1);
  eval_con_normals_inplace();

  while (true) {
    // cslsqp();
    cslsqp(&m_, &meq_, &la_, &n_, x_.data(), xl.data(), xu.data(), &fx_,
           c_.data(), g_.data(), a_.data(), &acc, &majiter_, &mode_, w.data(),
           &len_w, jw.data(), &len_jw, &alpha, &f0, &gs, &h1, &h2, &h3, &h4, &t,
           &t0, &to1, &iexact, &incons, &ireset, &itermx, &line, &n1, &n2, &n3);
    bool stopped = false;
    if (mode_ == 1) {
      fx_ = i_.func(x_);
      eval_con_inplace();
    } else if (mode_ == -1) {
      eval_g_inplace();
      eval_con_normals_inplace();
    } else {
      stopped = true;
    }

    if (stopped) {
      break;
    } else {
      majiter_prev_ = majiter_;
    }
  }
}

void slsqp_ws_t::eval_g_inplace() {
  g_ = i_.jac(x_);
  g_.resize(g_.size() + 1);
  g_[g_.size() - 1] = 0.0;
}

void slsqp_ws_t::eval_con_inplace() {
  for (size_t i = 0; i < i_.cons.size(); ++i) {
    auto& con = i_.cons[i];
    c_[i] = con.f_(x_);
  }
}

void slsqp_ws_t::eval_con_normals_inplace() {
  a_.setZero();
  for (size_t i = 0; i < i_.cons.size(); ++i) {
    auto &con = i_.cons[i];
    dynvec_t j = con.jac_(x_);
    if (j.size() != n_) {
      EXIT_ERR("constraint [%lu] return jac result with size=%lu, but there are %d "
               "independent vars",
               i, j.size(), n_);
    }
    a_.row(i).head(n_) = j;
  }
}

const char* slsqp_ws_t::retstr() const {
if (!run_) {
return "Optimization not started yet!";
}

switch(mode_) {
case -1: return "Gradient evaluation required (g & a)";
case 0: return "Optimization terminated successfully";
case 1: return "Function evaluation required (f & c)";
case 2: return "More equality constraints than independent variables";
case 3: return "More than 3*n iterations in LSQ subproblem";
case 4: return "Inequality constraints incompatible";
case 5: return "Singular matrix E in LSQ subproblem";
case 6: return "Singular matrix C in LSQ subproblem";
case 7: return "Rank-deficient equality constraint subproblem HFTI";
case 8: return "Positive directional derivative for linesearch";
case 9: return "Iteration limit reached";
}
return "unexpected mode!!!";
}
